package com.starbuds.server.api.family.service.admin.impl;

import com.starbuds.server.api.family.common.pojo.admin.FamilyAdminInfoVO;
import com.starbuds.server.api.family.common.pojo.db.FamilyAdminDO;
import com.starbuds.server.api.family.common.pojo.db.FamilyAdminSecurityDO;
import com.starbuds.server.api.family.common.service.admin.FaAdminService;
import com.starbuds.server.api.family.service.admin.dao.FamilyAdminDAO;
import com.starbuds.server.api.family.service.admin.dao.FamilyAdminSecurityDAO;
import com.starbuds.server.api.family.service.admin.dao.FamilyDAO;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.pojo.api.PageList;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Optional;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/1/16 13:59
 * @Description 管理员
 */
@Slf4j
@Service
public class FaAdminServiceImpl implements FaAdminService {

    @Resource
    FamilyAdminDAO familyAdminDAO;

    @Resource
    FamilyAdminSecurityDAO familyAdminSecurityDAO;

    @Resource
    FamilyDAO familyDAO;

    /**
     * 获取管理员信息
     *
     * @param adminId
     * @return
     */
    @Override
    public FamilyAdminInfoVO getInfo(Long adminId) {
        return familyAdminDAO.selectVOById(adminId);
    }

    /**
     * 判断管理员是否存在
     *
     * @param adminId 管理员id
     * @return
     */
    @Override
    public boolean isAdminExist(Long adminId) {
        return familyAdminDAO.countById(adminId) == 1;
    }

    /**
     * 创建管理员
     *
     * @param adminDO
     * @param encryptPassword
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean createAdmin(FamilyAdminDO adminDO, String encryptPassword) {
        if (familyAdminDAO.insert(adminDO) != 1) {
            throw new RuntimeException("创建管理员失败");
        }
        FamilyAdminSecurityDO securityDO = new FamilyAdminSecurityDO();
        securityDO.setAdminId(adminDO.getAdminId());
        securityDO.setAdminAccount(adminDO.getAdminPhone());
        securityDO.setAdminIsLocked(0);
        securityDO.setRoleId(0L);
        securityDO.setAdminPass(encryptPassword);
        securityDO.setStatus(CommonStatus.Normal);
        if (familyAdminSecurityDAO.insert(securityDO) != 1) {
            throw new RuntimeException("保存密码失败");
        }
        return true;
    }

    /**
     * 编辑管理员
     *
     * @param adminDO 管理员
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean editAdmin(FamilyAdminDO adminDO) {
        return familyAdminDAO.updateById(adminDO) > 0;
    }

    /**
     * 设置管理员状态
     *
     * @param adminId     管理员id
     * @param adminStatus
     * @return 是否成功
     */
    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean setAdminStatus(Long adminId, Integer adminStatus) {
        return familyAdminDAO.updateStatus(adminId, adminStatus) > 0;
    }

    /**
     * 获取管理员列表
     *
     * @param adminName   管理员姓名
     * @param adminPhone  手机号
     * @param adminStatus 状态
     * @param pageIndex   当前页码
     * @param pageSize    分页大小
     * @param familyId    家族id
     * @return result对象
     */
    @Override
    public PageList<FamilyAdminInfoVO> getAdminPageList(String adminName, String adminPhone, Integer adminStatus, int pageIndex, int pageSize, Long familyId) {
        PageList<FamilyAdminInfoVO> page = new PageList<>(pageIndex, pageSize);
        page.setList(familyAdminDAO.getAdminPageList(adminName, adminPhone, adminStatus, page.getPageCursor(), page.getPageSize(), familyId));
        page.setTotal(familyAdminDAO.countAdminPageList(adminName, adminPhone, adminStatus, familyId));
        return page;
    }

    /**
     * 判断手机号是否存在
     *
     * @param adminPhone 手机号
     * @return 是否存在
     */
    @Override
    public boolean isPhoneExist(String adminPhone) {
        return familyAdminSecurityDAO.countByPhone(adminPhone) > 0;
    }

    /**
     * 判断账号是否可用
     *
     * @param adminId
     * @return
     */
    @Override
    public boolean isAdminEnable(Long adminId) {
        return Optional.ofNullable(familyAdminDAO.getAdminStatus(adminId)).map(e -> CommonStatus.Normal == e).orElse(false);
    }

    /**
     * 获取家族id
     *
     * @param adminId
     * @return
     */
    @Override
    public Long getFamilyId(Long adminId) {
        return familyAdminDAO.selectFamilyId(adminId);
    }

    /**
     * 是否家族长
     *
     * @param adminId
     * @param familyId
     * @return
     */
    @Override
    public boolean isPresident(Long adminId, Long familyId) {
        return adminId.equals(familyDAO.selectPresident(familyId));
    }
}
