package com.starbuds.server.api.family.service.admin.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.starbuds.server.api.family.common.pojo.admin.FamilyWalletWithdrawHisVO;
import com.starbuds.server.api.family.common.service.admin.FaFamilyWalletWithdrawHisService;
import com.starbuds.server.api.family.service.admin.dao.*;
import com.starbuds.server.common.pojo.api.PageList;
import com.starbuds.server.common.pojo.db.FamilyWalletDO;
import com.starbuds.server.common.pojo.db.FamilyWalletHisDO;
import com.starbuds.server.common.pojo.db.FamilyWalletWithdrawHisDO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.util.List;
import java.util.Optional;

/**
 * @Author jiangxiaowei
 * @DateTime 2020/3/30 16:19
 * @Description 家族钱包提现记录业务实现
 */
@Slf4j
@Service
public class FaFamilyWalletWithdrawHisServiceImpl implements FaFamilyWalletWithdrawHisService {

    @Resource
    FamilyWalletWithdrawHisDAO familyWalletWithdrawHisDAO;

    @Resource
    FamilyWalletDAO familyWalletDAO;

    @Resource
    FamilyWalletHisDAO familyWalletHisDAO;

    @Resource
    FamilyBankcardDAO familyBankcardDAO;

    @Resource
    FamilyDAO familyDAO;

    @Resource
    AdminDAO adminDAO;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public BigDecimal addWithdrawApply(FamilyWalletWithdrawHisDO withdrawHisDO, String hisRemarks) {
        try {
            // 家族id
            Long familyId = withdrawHisDO.getFamilyId();
            // 检查账户
            FamilyWalletDO walletDO = checkWallet(familyId);
            // 提现金额
            BigDecimal balanceChange = withdrawHisDO.getAmount();
            // 原有余额
            BigDecimal balanceOld = walletDO.getBalance();
            // 原有版本号
            Long version = walletDO.getVersion();
            // 更新余额
            BigDecimal balanceNew = updateBalance(familyId, balanceOld, version, balanceChange);
            if (balanceNew.compareTo(BigDecimal.ZERO) == -1) {
                throw new RuntimeException("余额不足");
            }
            // 写入记录
            if (familyWalletWithdrawHisDAO.insert(withdrawHisDO) == 1) {
                FamilyWalletHisDO walletHisDO = new FamilyWalletHisDO();
                walletHisDO.setHisId(withdrawHisDO.getHisId());
                walletHisDO.setFamilyId(familyId);
                walletHisDO.setBalanceOld(walletDO.getBalance());
                walletHisDO.setBalanceValue(withdrawHisDO.getAmount().compareTo(BigDecimal.ZERO) > 0 ? withdrawHisDO.getAmount().negate() : withdrawHisDO.getAmount());
                walletHisDO.setBalanceNew(balanceNew);
                walletHisDO.setCreateTime(System.currentTimeMillis());
                walletHisDO.setHisRemarks(hisRemarks);
                walletHisDO.setOrderId(withdrawHisDO.getHisId());
                familyWalletHisDAO.insert(walletHisDO);
                return balanceNew;
            }
        } catch (Exception e) {
            log.info("更新失败{}", e);
            TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
        }
        return null;
    }

    @Override
    public BigDecimal getAmount(Long familyId) {
        return Optional.ofNullable(familyWalletDAO.selectBalance(familyId)).orElse(BigDecimal.ZERO);
    }

    @Override
    public Integer getApplyStatus(Long familyId) {
        return familyWalletWithdrawHisDAO.selectLastApplyStatus(familyId);
    }

    @Override
    public PageList<FamilyWalletWithdrawHisVO> getFamilyWithdrawHisPageList(Long familyId, Integer applyStatus, Long startTime, Long endTime, int pageIndex, int pageSize) {
        PageList<FamilyWalletWithdrawHisVO> page = new PageList<>(pageIndex, pageSize);
        List<FamilyWalletWithdrawHisVO> list = familyWalletWithdrawHisDAO.getFamilyWithdrawHisPageList(familyId, applyStatus, startTime, endTime, page.getPageCursor(), pageSize);
        if (!CollectionUtil.isEmpty(list)) {
            for (FamilyWalletWithdrawHisVO vo : list) {
                vo.setFamilyName(familyDAO.selectFamilyName(vo.getFamilyId()));
                vo.setBankcard(familyBankcardDAO.selectById(vo.getBankcardId()));
                vo.setApplyAdminName(adminDAO.selectAdminName(vo.getApplyAdminId()));
            }
        }
        page.setList(list);
        page.setTotal(familyWalletWithdrawHisDAO.countFamilyWithdrawHisPageList(familyId, applyStatus, startTime, endTime));
        return page;
    }


    /**
     * 更新余额
     *
     * @param familyId
     * @param balance
     * @param version
     * @param changeBalance
     * @return
     */
    private BigDecimal updateBalance(Long familyId, BigDecimal balance, Long version, BigDecimal changeBalance) {
        FamilyWalletDO walletUpdate = new FamilyWalletDO();
        walletUpdate.setFamilyId(familyId);
        walletUpdate.setUpdateTime(System.currentTimeMillis());
        walletUpdate.setBalance(balance.add(changeBalance.negate()));
        walletUpdate.setVersion(version);
        // 扣钱CAS
        if (familyWalletDAO.updateBalanceCas(walletUpdate) == 0) {
            FamilyWalletDO walletDO = familyWalletDAO.selectById(familyId);
            return updateBalance(familyId, walletDO.getBalance(), walletDO.getVersion(), changeBalance);
        }
        return walletUpdate.getBalance();
    }

    /**
     * 检查账户
     *
     * @param familyId
     */
    private FamilyWalletDO checkWallet(Long familyId) {
        // 判断账户是否存在
        FamilyWalletDO familyWalletDO = familyWalletDAO.selectById(familyId);
        // 无则初始化
        if (familyWalletDO == null) {
            familyWalletDO = new FamilyWalletDO();
            familyWalletDO.setFamilyId(familyId);
            familyWalletDO.setBalance(BigDecimal.ZERO);
            familyWalletDO.setVersion(0L);
            familyWalletDO.setUpdateTime(System.currentTimeMillis());
            try {
                familyWalletDAO.insert(familyWalletDO);
            } catch (Exception e) {
                checkWallet(familyId);
            }
        }
        return familyWalletDO;
    }
}
