package com.starbuds.server.api.family.v1.impl;

import cn.hutool.core.util.RandomUtil;
import com.starbuds.server.api.family.common.pojo.admin.TokenParam;
import com.starbuds.server.api.family.common.pojo.db.FamilyAdminDO;
import com.starbuds.server.api.family.common.pojo.db.FamilyAdminSecurityDO;
import com.starbuds.server.api.family.common.service.admin.FaAdminSecurityService;
import com.starbuds.server.api.family.common.service.admin.FaAdminService;
import com.starbuds.server.api.family.inf.PermitGuest;
import com.starbuds.server.api.family.service.admin.impl.FaAdminSecurityServiceImpl;
import com.starbuds.server.api.family.v1.facade.AdminFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.service.core.IdentityService;
import lombok.extern.slf4j.Slf4j;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.*;

/**
 * 管理员管理
 *
 * @author jiangxiaowei
 * @date 2020/1/16 14:05
 */
@Path("/v1/admin")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class AdminFacadeImpl extends BaseFacade implements AdminFacade {

    @Resource
    FaAdminSecurityService faAdminSecurityService;

    @Resource
    FaAdminService faAdminService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    /**
     * 账号密码登录
     *
     * @param account
     * @param password
     * @return
     */
    @POST
    @Path("login")
    @PermitGuest
    @Override
    public Result login(@FormParam("account") String account,
                        @FormParam("password") String password) {

        FamilyAdminSecurityDO adminSecurityDO = faAdminSecurityService.getSecurityByAccount(account);
        if (adminSecurityDO == null || !faAdminSecurityService.checkPass(password, adminSecurityDO.getAdminPass())) {
            return new Result(GlobalApiError.OperationFailed, "账号或密码错误");
        }

        if (!faAdminService.isAdminEnable(adminSecurityDO.getAdminId())) {
            return new Result(GlobalApiError.OperationFailed, "该账号已被停用");
        }

        // 创建令牌登录
        String token = RandomUtil.randomString(64);
        if (!faAdminSecurityService.setToken(adminSecurityDO.getAdminId(), this.getHeader().getClientType().getValue(), token)) {
            return new Result(GlobalApiError.OperationFailed, "令牌创建失败");
        }
        // 加密令牌
        TokenParam tokenParam = new TokenParam();
        tokenParam.setAdminId(adminSecurityDO.getAdminId());
        tokenParam.setRoleId(adminSecurityDO.getRoleId());
        tokenParam.setToken(token);
        tokenParam.setTimestamp(System.currentTimeMillis());
        tokenParam.setFamilyId(faAdminService.getFamilyId(adminSecurityDO.getAdminId()));
        String tokenEncrypted = faAdminSecurityService.encryptToken(tokenParam);

        MutableResult mResult = new MutableResult();
        mResult.put("token", tokenEncrypted);
        return mResult.build();
    }

    /**
     * 个人信息
     *
     * @return
     */
    @GET
    @Path("getInfo")
    @Override
    public Result getInfo() {
        return new Result(faAdminService.getInfo(this.getToken().getAdminId()));
    }

    /**
     * 重置密码
     *
     * @param adminId 被重置人的id
     * @param newPwd  新密码
     * @return result对象
     */
    @POST
    @Path("resetPwd")
    @Override
    public Result resetPwd(@FormParam("adminId") Long adminId,
                           @FormParam("newPwd") String newPwd) {
        if (!faAdminService.isAdminExist(adminId)) {
            return new Result(GlobalApiError.OperationFailed, "管理员不存在");
        }

        // 判断是否家族长
        if (faAdminService.isPresident(adminId, this.getToken().getFamilyId())) {
            if (!this.getToken().getAdminId().equals(adminId)) {
                return new Result(GlobalApiError.OperationFailed, "权限不足，不可操作家族长");
            }
        }

        if (faAdminSecurityService.changePwd(adminId, newPwd)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 修改密码
     *
     * @param oldPwd 旧密码
     * @param newPwd 新密码
     * @return result对象
     */
    @POST
    @Path("changePwd")
    @Override
    public Result changePwd(@FormParam("oldPwd") String oldPwd,
                            @FormParam("newPwd") String newPwd) {
        FamilyAdminSecurityDO dbSecurity = faAdminSecurityService.getSecurityByAdminId(this.getToken().getAdminId());
        //校验旧密码
        if (dbSecurity == null || !faAdminSecurityService.checkPass(oldPwd, dbSecurity.getAdminPass())) {
            return new Result(GlobalApiError.OperationFailed, "旧密码不正确");
        }
        if (faAdminSecurityService.changePwd(this.getToken().getAdminId(), newPwd)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 创建管理员
     *
     * @param adminName   管理员名称
     * @param adminPhone  手机号
     * @param adminAvatar 头像
     * @param adminEmail  email
     * @param adminPass   密码
     * @param adminStatus 状态
     * @return result对象
     */
    @POST
    @Path("createAdmin")
    @Override
    public Result createAdmin(@FormParam("adminName") String adminName,
                              @FormParam("adminPhone") String adminPhone,
                              @FormParam("adminAvatar") String adminAvatar,
                              @FormParam("adminEmail") String adminEmail,
                              @FormParam("adminPass") String adminPass,
                              @FormParam("adminStatus") Integer adminStatus) {
        if (faAdminService.isPhoneExist(adminPhone)) {
            return new Result(GlobalApiError.OperationFailed, "该手机号已是管理员");
        }
        FamilyAdminDO adminDO = new FamilyAdminDO();
        adminDO.setAdminId(identityService.getId());
        adminDO.setAdminName(adminName);
        adminDO.setAdminPhone(adminPhone);
        adminDO.setAdminAvatar(adminAvatar);
        adminDO.setAdminCreateTime(System.currentTimeMillis());
        adminDO.setAdminEmail(adminEmail);
        adminDO.setAdminStatus(adminStatus);
        adminDO.setFamilyId(this.getToken().getFamilyId());

        String encryptPassword = FaAdminSecurityServiceImpl.encryptPassword(adminPass);

        if (faAdminService.createAdmin(adminDO, encryptPassword)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 编辑管理员
     *
     * @param adminId     管理员id
     * @param adminName   管理员名称
     * @param adminAvatar 头像
     * @param adminEmail  email
     * @param adminStatus 状态
     * @return result对象
     */
    @POST
    @Path("editAdmin")
    @Override
    public Result editAdmin(@FormParam("adminId") Long adminId,
                            @FormParam("adminName") String adminName,
                            @FormParam("adminAvatar") String adminAvatar,
                            @FormParam("adminEmail") String adminEmail,
                            @FormParam("adminStatus") Integer adminStatus) {
        if (!faAdminService.isAdminExist(adminId)) {
            return new Result(GlobalApiError.OperationFailed, "管理员不存在");
        }
        FamilyAdminDO adminDO = new FamilyAdminDO();
        adminDO.setAdminId(adminId);
        adminDO.setAdminName(adminName);
        adminDO.setAdminAvatar(adminAvatar);
        adminDO.setAdminEmail(adminEmail);
        adminDO.setAdminStatus(adminStatus);
        if (faAdminService.editAdmin(adminDO)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 设置管理员状态
     *
     * @param adminId
     * @param adminStatus
     * @return result对象
     */
    @POST
    @Path("setAdminStatus")
    @Override
    public Result setAdminStatus(@FormParam("adminId") Long adminId, @FormParam("adminStatus") Integer adminStatus) {
        if (!faAdminService.isAdminExist(adminId)) {
            return new Result(GlobalApiError.OperationFailed, "管理员不存在");
        }

        // 判断是否家族长
        if (faAdminService.isPresident(adminId, this.getToken().getFamilyId())) {
            if (!this.getToken().getAdminId().equals(adminId)) {
                return new Result(GlobalApiError.OperationFailed, "权限不足，不可操作家族长");
            }
        }

        if (adminStatus == null || adminStatus > 1 || adminStatus < -1) {
            return new Result(GlobalApiError.ServerError, "非法参数");
        }
        if (faAdminService.setAdminStatus(adminId, adminStatus)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取管理员列表
     *
     * @param adminName   管理员姓名
     * @param adminPhone  手机号
     * @param adminStatus 状态
     * @param pageIndex   当前页码
     * @param pageSize    分页大小
     * @return result对象
     */
    @GET
    @Path("getAdminPageList")
    @Override
    public Result getAdminPageList(@QueryParam("adminName") String adminName,
                                   @QueryParam("adminPhone") String adminPhone,
                                   @QueryParam("adminStatus") Integer adminStatus,
                                   @QueryParam("pageIndex") int pageIndex,
                                   @QueryParam("pageSize") int pageSize) {
        return new Result(faAdminService.getAdminPageList(adminName, adminPhone, adminStatus, pageIndex, pageSize, this.getToken().getFamilyId()));
    }

}
