package com.starbuds.server.api.family.v1.impl;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.api.family.common.service.admin.FaFamilyService;
import com.starbuds.server.api.family.v1.facade.FamilyFacade;
import com.starbuds.server.common.constant.common.BoolIntVal;
import com.starbuds.server.common.constant.common.CommonStatus;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.data.PlayTopTypeEnum;
import com.starbuds.server.common.constant.message.im.ImActionType;
import com.starbuds.server.common.constant.message.push.ModuleTypeEnum;
import com.starbuds.server.common.constant.user.ApplyTypeEnum;
import com.starbuds.server.common.constant.user.AuditStatusEnum;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.*;
import com.starbuds.server.common.pojo.message.im.biz.SystemMsg;
import com.starbuds.server.common.service.core.IdentityService;
import com.starbuds.server.common.service.data.DataService;
import com.starbuds.server.common.service.data.SkillService;
import com.starbuds.server.common.service.message.ImService;
import com.starbuds.server.common.service.user.FamilyService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.*;
import java.util.Arrays;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 家族业务实现
 *
 * @author iangxiaowei
 * @date 2020/1/15 19:07
 */
@Path("/v1/family")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class FamilyFacadeImpl extends BaseFacade implements FamilyFacade {

    @Resource
    FaFamilyService faFamilyService;

    @Reference(version = "1.0.0")
    FamilyService familyService;

    @Reference(version = "1.0.0")
    SkillService skillService;

    @Reference(version = "1.0.0")
    IdentityService identityService;

    @Reference(version = "1.0.0")
    DataService dataService;

    @Reference(version = "1.0.0")
    ImService imService;

    /**
     * 获取家族申请列表
     *
     * @param applyType
     * @param userId
     * @param auditStatus
     * @param userName
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @GET
    @Path("getFamilyAuditPageList")
    @Override
    public Result getFamilyAuditPageList(@QueryParam("applyType") Integer applyType,
                                         @QueryParam("userId") Long userId,
                                         @QueryParam("auditStatus") Integer auditStatus,
                                         @QueryParam("userName") String userName,
                                         @QueryParam("pageIndex") int pageIndex,
                                         @QueryParam("pageSize") int pageSize) {
        return new Result(faFamilyService.getFamilyAuditPageList(this.getToken().getFamilyId(), applyType, userId, auditStatus, userName, pageIndex, pageSize));
    }

    /**
     * 获取家族技能认证申请
     *
     * @param userId
     * @param auditStatus
     * @param pageIndex
     * @param pageSize
     * @return
     */
    @GET
    @Path("getFamilySkillAuditPageList")
    @Override
    public Result getFamilySkillAuditPageList(@QueryParam("userId") Long userId,
                                              @QueryParam("auditStatus") Integer auditStatus,
                                              @QueryParam("pageIndex") int pageIndex,
                                              @QueryParam("pageSize") int pageSize) {
        return new Result(faFamilyService.getFamilySkillAuditPageList(this.getToken().getFamilyId(), userId, auditStatus, pageIndex, pageSize));
    }

    /**
     * 设置家族签约状态
     *
     * @param auditId
     * @param auditStatus
     * @param auditMsg
     * @param userRatio
     * @return
     */
    @POST
    @Path("setFamilyAuditStatus")
    @Override
    public Result setFamilyAuditStatus(@FormParam("auditId") Long auditId,
                                       @FormParam("auditStatus") Integer auditStatus,
                                       @FormParam("auditMsg") String auditMsg,
                                       @FormParam("userRatio") Double userRatio) {
        FamilyAuditDO auditDO = faFamilyService.getFamilyAudit(auditId);
        if (auditDO == null) {
            return new Result(GlobalApiError.OperationFailed, "记录不存在");
        }
        AuditStatusEnum auditStatusEnum = AuditStatusEnum.toEnum(auditStatus);
        if (auditStatusEnum == null) {
            return new Result(GlobalApiError.ServerError, "非法状态");
        }
        if (auditDO.getAuditStatus() != AuditStatusEnum.WAIT.getCode()) {
            return new Result(GlobalApiError.OperationFailed, "申请已处理");
        }
        long curTime = System.currentTimeMillis();

        // 是否强制解约
        boolean isForce = Optional.ofNullable(auditDO.getIsForce()).map(e -> BoolIntVal.True == e).orElse(false);
        ApplyTypeEnum applyTypeEnum = ApplyTypeEnum.toEnum(auditDO.getApplyType());
        if (isForce && applyTypeEnum == ApplyTypeEnum.LEAVE) {
            return new Result(GlobalApiError.OperationFailed, "无法操作强制解约流程");
        }

        // 设置审核状态
        if (faFamilyService.setFamilyAuditStatus(auditId, auditStatus, auditMsg, curTime, this.getToken().getAdminId())) {

            Long userId = auditDO.getUserId();
            Long familyId = auditDO.getFamilyId();
            Long adminId = this.getToken().getAdminId();

            if (AuditStatusEnum.PASS.getCode() == auditStatus) {
                FamilyUserDO userDO = new FamilyUserDO();
                userDO.setUserId(userId);
                userDO.setFamilyId(familyId);
                switch (applyTypeEnum) {
                    case JOIN:
                        userDO.setSignTime(curTime);
                        userDO.setEndTime(-1L);
                        userDO.setStatus(CommonStatus.Normal);
                        userDO.setRatio(userRatio != null ? userRatio : faFamilyService.getUserRatio(familyId));
                        break;
                    case LEAVE:
                        userDO.setSignTime(null);
                        userDO.setEndTime(null);
                        userDO.setStatus(CommonStatus.Deleted);
                        userDO.setRatio(null);
                        // 作废技能
                        faFamilyService.setSkillStatus(familyId, userId, CommonStatus.Deleted, curTime);
                        faFamilyService.setSkillAuditStatus(familyId, userId, AuditStatusEnum.DENY.getCode(), "离开家族「自动拒绝审核技能」", adminId, curTime);
                        break;
                    default:
                        break;
                }
                if (!faFamilyService.addFamilyUser(userDO, applyTypeEnum)) {
                    return new Result(GlobalApiError.ServerError);
                }

            } else {

                switch (applyTypeEnum) {
                    // 拒绝家族时，作废技能
                    case JOIN:
                        faFamilyService.setSkillStatus(familyId, userId, CommonStatus.Deleted, curTime);
                        faFamilyService.setSkillAuditStatus(familyId, userId, CommonStatus.Deleted, "家族签约被拒绝", adminId, curTime);
                        break;
                    case LEAVE:
                        break;
                    default:
                        break;
                }
            }
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 设置家族技能认证状态
     *
     * @param auditId
     * @param auditStatus
     * @param auditMsg
     * @return
     */
    @POST
    @Path("setFamilySkillAuditStatus")
    @Override
    public Result setFamilySkillAuditStatus(@FormParam("auditId") Long auditId, @FormParam("auditStatus") Integer auditStatus, @FormParam("auditMsg") String auditMsg) {
        FamilySkillAuditDO auditDO = faFamilyService.getFamilySkillAudit(auditId);
        if (auditDO == null) {
            return new Result(GlobalApiError.ServerError, "认证申请不存在");
        }
        if (auditDO.getAuditStatus() != AuditStatusEnum.WAIT.getCode()) {
            return new Result(GlobalApiError.ServerError, "认证申请已处理");
        }
        AuditStatusEnum statusEnum = AuditStatusEnum.toEnum(auditStatus);
        if (statusEnum == null) {
            return new Result(GlobalApiError.ServerError, "传入状态非法");
        }

        long curTime = System.currentTimeMillis();
        Long adminId = this.getToken().getAdminId();

        // 审核技能
        if (faFamilyService.setFamilySkillAuditStatus(auditId, statusEnum.getCode(), auditMsg, adminId, curTime)) {
            if (AuditStatusEnum.PASS.getCode() == auditStatus) {
                FamilySkillDO skillDO = new FamilySkillDO();
                skillDO.setUserId(auditDO.getUserId());
                skillDO.setFamilyId(auditDO.getFamilyId());
                skillDO.setSkillId(auditDO.getSkillId());
                skillDO.setSkillTagId(auditDO.getSkillTagId());
                skillDO.setOrderStatus(BoolIntVal.True);
                skillDO.setOrderPrice("语音速配".equals(skillService.getSkillName(skillDO.getSkillId())) ? 100L : 490L);
                skillDO.setOrderTime("00:00-23:59");
                skillDO.setSkillStatus(CommonStatus.Normal);
                skillDO.setUpdateTime(curTime);
                skillDO.setOrderCount(0);
                faFamilyService.addSkill(skillDO);

                // 添加陪玩头条
                PlayTopDO topDO = new PlayTopDO();
                topDO.setTopId(identityService.getId());
                topDO.setTopType(PlayTopTypeEnum.SKILLS_CERTIFICATION.getCode());
                topDO.setUserId(auditDO.getUserId());
                topDO.setSkillId(auditDO.getSkillId());
                topDO.setCreateTime(curTime);
                dataService.addPlayTop(topDO);

                // 如果家族申请等待审核
                if (faFamilyService.isFamilyApplyWaitAudit(auditDO.getFamilyId(), auditDO.getUserId(), ApplyTypeEnum.JOIN.getCode())) {
                    // 通过家族审核
                    faFamilyService.setFamilyAuditStatus(auditDO.getFamilyId(), auditDO.getUserId(), ApplyTypeEnum.JOIN.getCode(), AuditStatusEnum.PASS.getCode(), "技能认证通过", curTime, adminId);
                    FamilyUserDO userDO = new FamilyUserDO();
                    userDO.setUserId(auditDO.getUserId());
                    userDO.setFamilyId(auditDO.getFamilyId());
                    userDO.setSignTime(curTime);
                    userDO.setEndTime(-1L);
                    userDO.setStatus(CommonStatus.Normal);
                    faFamilyService.addFamilyUser(userDO, ApplyTypeEnum.JOIN);
                }
            } else {
                // 未通过
                try {
                    // 系统消息
                    SystemMsg systemMsg = new SystemMsg();
                    systemMsg.setContentType(com.starbuds.server.common.constant.message.im.ContentType.TEXT);
                    systemMsg.setTitle("技能认证未通过");
                    String body = String.format("你的【%s】技能认证由于%s%s%s原因，未通过认证，请修改后重新认证。", skillService.getSkillName(auditDO.getSkillId()), '"', auditMsg, '"');
                    systemMsg.setBody(body);
                    systemMsg.setActionType(ImActionType.JUMP_NATIVE.getValue());
                    systemMsg.setModuleType(ModuleTypeEnum.SKILL_AUTH.getDesc());
                    systemMsg.setModuleId(Convert.toStr(1)); // 认证中
                    imService.sendCustomImMsg(null, auditDO.getUserId(), systemMsg, null);
                } catch (Exception e) {
                    log.error("出现错误", e);
                }
            }
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 获取家族详情
     *
     * @return
     */
    @GET
    @Path("getFamilyDetail")
    @Override
    public Result getFamilyDetail() {
        return new Result(faFamilyService.getFamilyDetail(this.getToken().getFamilyId()));
    }

    /**
     * 获取家族默认用户分成比例
     *
     * @return
     */
    @GET
    @Path("getUserRatio")
    @Override
    public Result getUserRatio() {
        return new Result(faFamilyService.getUserRatio(this.getToken().getFamilyId()));
    }

    /**
     * 设置家族用户分成比例
     *
     * @param userId
     * @param ratio
     * @return
     */
    @POST
    @Path("setUserRatio")
    @Override
    public Result setUserRatio(@FormParam("userId") Long userId, @FormParam("ratio") Double ratio) {
        Long familyId = this.getToken().getFamilyId();
        if (!familyService.isFamilyUserExist(familyId, userId)) {
            return new Result(GlobalApiError.ServerError, "无权操作");
        }
        if (faFamilyService.setUserRatio(familyId, userId, ratio)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取技能订单列表
     *
     * @param pageIndex
     * @param pageSize
     * @param payStatus
     * @param orderStatus
     * @return
     */
    @GET
    @Path("getSkillOrderPageList")
    @Override
    public Result getSkillOrderPageList(@QueryParam("pageIndex") int pageIndex,
                                        @QueryParam("pageSize") int pageSize,
                                        @QueryParam("payStatus") Integer payStatus,
                                        @QueryParam("orderStatus") Integer orderStatus) {
        return new Result(faFamilyService.getSkillOrderPageList(pageIndex, pageSize, this.getToken().getFamilyId(), payStatus, orderStatus));
    }

    /**
     * 移除技能
     *
     * @param auditId
     * @param reason
     * @return
     */
    @POST
    @Path("removeSkill")
    @Override
    public Result removeSkill(@FormParam("auditId") Long auditId, @FormParam("reason") String reason) {
        FamilySkillAuditDO skillAudit = faFamilyService.getFamilySkillAudit(auditId);
        if (skillAudit == null) {
            return new Result(GlobalApiError.ServerError, "技能认证记录不存在");
        }
        Integer auditStatus = skillAudit.getAuditStatus();
        if (auditStatus != AuditStatusEnum.PASS.getCode()) {
            return new Result(GlobalApiError.ServerError, "仅可对通过的申请操作");
        }

        Long familyId = this.getToken().getFamilyId();

        if (!familyId.equals(skillAudit.getFamilyId())) {
            return new Result(GlobalApiError.ServerError, "仅能处理当前家族的认证记录");
        }

        Long userId = skillAudit.getUserId();
        Long skillId = skillAudit.getSkillId();

        if (!faFamilyService.isSkillAuth(familyId, userId, skillId)) {
            log.info("未认证无需重新审核");
            return new Result();
        }

        if (faFamilyService.removeSkill(auditId, reason, this.getToken().getAdminId())) {
            return new Result();
        }
        return new Result(GlobalApiError.ServerError);
    }

    /**
     * 获取家族成员
     *
     * @param pageIndex
     * @param pageSize
     * @param userName
     * @return
     */
    @GET
    @Path("getFamilyUserList")
    @Override
    public Result getFamilyUserList(@QueryParam("pageIndex") int pageIndex, @QueryParam("pageSize") int pageSize, @QueryParam("userName") String userName) {
        return new Result(faFamilyService.getFamilyUserList(this.getToken().getFamilyId(), pageIndex, pageSize, userName));
    }

    /**
     * 解约用户（批量）
     *
     * @param userIds
     * @param content
     * @return
     */
    @POST
    @Path("setTerminate")
    @Override
    public Result setTerminate(@FormParam("userIds") String userIds,
                               @FormParam("content") String content) {
        if (StringUtils.isEmpty(userIds) || StringUtils.isEmpty(content)) {
            return new Result(GlobalApiError.OperationFailed, "缺少必要参数");
        }
        Set<Long> userIdSet = Arrays.stream(userIds.split(",")).map(Convert::toLong).collect(Collectors.toSet());
        Long familyId = this.getToken().getFamilyId();
        Long president = faFamilyService.getPresident(familyId);
        if (userIdSet.contains(president)) {
            return new Result(GlobalApiError.OperationFailed, "不能解约家族长");
        }
        if (faFamilyService.setTerminate(familyId, this.getToken().getAdminId(), userIdSet, content)) {
            return new Result();
        }
        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取签约记录
     *
     * @param pageIndex
     * @param pageSize
     * @param userId
     * @return
     */
    @GET
    @Path("getUserHisList")
    @Override
    public Result getUserHisList(@QueryParam("pageIndex") int pageIndex, @QueryParam("pageSize") int pageSize, @QueryParam("userId") Long userId) {
        return new Result(faFamilyService.getUserHisList(pageIndex, pageSize, userId));
    }
}
