package com.starbuds.server.api.family.v1.impl;

import cn.hutool.core.convert.Convert;
import com.starbuds.server.api.family.common.service.admin.FaFamilyService;
import com.starbuds.server.api.family.common.service.admin.FaRtcRoomService;
import com.starbuds.server.api.family.v1.facade.RtcFacade;
import com.starbuds.server.common.constant.common.GlobalApiError;
import com.starbuds.server.common.constant.rtc.RoomTypeEnum;
import com.starbuds.server.common.constant.rtc.RtcRoomRole;
import com.starbuds.server.common.pojo.api.MutableResult;
import com.starbuds.server.common.pojo.api.Result;
import com.starbuds.server.common.pojo.db.UserDO;
import com.starbuds.server.common.pojo.rtc.AuthRtcConfig;
import com.starbuds.server.common.pojo.rtc.RtcRoomAddDTO;
import com.starbuds.server.common.pojo.rtc.RtcRoomEditDTO;
import com.starbuds.server.common.service.data.DataService;
import com.starbuds.server.common.service.rtc.RtcRoomService;
import com.starbuds.server.common.service.user.RtcUserService;
import com.starbuds.server.common.service.user.UserService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.dubbo.config.annotation.Reference;
import org.apache.dubbo.rpc.protocol.rest.support.ContentType;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.ws.rs.*;

/**
 * 语音接口
 *
 * @author jiangxiaowei
 * @date 2020/11/22 18:52
 */
@Path("/v1/rtc")
@Produces(ContentType.APPLICATION_JSON_UTF_8)
@Component
@Slf4j
public class RtcFacadeImpl extends BaseFacade implements RtcFacade {

    @Resource
    FaRtcRoomService faRtcRoomService;

    @Resource
    FaFamilyService faFamilyService;

    @Reference(version = "1.0.0")
    DataService dataService;

    @Reference(version = "1.0.0")
    RtcUserService rtcUserService;

    @Reference(version = "1.0.0")
    RtcRoomService rtcRoomService;

    @Reference(version = "1.0.0")
    private UserService userService;

    /**
     * 创建房间
     *
     * @param roomType
     * @param categoryId
     * @param roomName
     * @param roomCover
     * @return
     */
    @POST
    @Path("createRoom")
    @Override
    public Result createRoom(@FormParam("roomType") Integer roomType,
                             @FormParam("categoryId") Long categoryId,
                             @FormParam("roomName") String roomName,
                             @FormParam("roomCover") String roomCover) {
        // 判断房间类型
        if (RoomTypeEnum.toEnum(roomType) == null) {
            return new Result(GlobalApiError.OperationFailed, "不支持该房间类型");
        }

        // 判断类目是否存在
        if (!dataService.isRtcRoomCategoryExists(categoryId)) {
            return new Result(GlobalApiError.OperationFailed, "无法设置该标签");
        }

        // 判断房间名称
        if (StringUtils.isBlank(roomName)) {
            return new Result(GlobalApiError.OperationFailed, "请填写房间名称");
        }

        Long familyId = this.getToken().getFamilyId();
        // 授权语聊配置
        AuthRtcConfig authRtcConfig = rtcUserService.getAuthRtcConfig(familyId);
        if (authRtcConfig == null) {
            return new Result(GlobalApiError.ServerError, "未读取到家族语音房配置，请联系管理员");
        }

        // 家族开通房间数
        int roomCount = rtcRoomService.getRoomCountByFamilyId(familyId);
        if (roomCount >= authRtcConfig.getRoomNum()) {
            return new Result(GlobalApiError.ServerError, "家族房间数量超上限");
        }

        // 房间分类
        String authRoomCategory = authRtcConfig.getRoomCategory();
        if (!authRoomCategory.contains(Convert.toStr(categoryId))) {
            return new Result(GlobalApiError.ServerError, "家族房间未获得该类目授权");
        }

        // 房间类型
        String authRoomType = authRtcConfig.getRoomType();
        if (!authRoomType.contains(Convert.toStr(roomType))) {
            return new Result(GlobalApiError.ServerError, "家族房间未获得该类型授权");
        }

        Long presidentUserId = faFamilyService.getPresident(familyId);
        Long channelId = this.getHeader().getChannelId();
        if (channelId == null) {
            UserDO user = userService.selectById(presidentUserId);
            if (user != null) {
                // channelId 为空则不会记录统计数据
                channelId = user.getChannelId();
            }
        }

        RtcRoomAddDTO rtcRoomAddDTO = new RtcRoomAddDTO();
        rtcRoomAddDTO.setUserId(presidentUserId);
        rtcRoomAddDTO.setCategoryId(categoryId);
        rtcRoomAddDTO.setRoomCover(roomCover);
        rtcRoomAddDTO.setRoomName(roomName);
        rtcRoomAddDTO.setRoomType(roomType);
        rtcRoomAddDTO.setChannelId(channelId);
        rtcRoomAddDTO.setFamilyId(familyId);

        Long roomId = rtcRoomService.addRoom(rtcRoomAddDTO);
        if (roomId == null) {
            return new Result(GlobalApiError.OperationFailed, "房间创建失败");
        }

        MutableResult mResult = new MutableResult();
        mResult.put("roomId", roomId);
        return mResult.build();
    }

    /**
     * 编辑房间
     *
     * @param roomId
     * @param roomCover
     * @param roomHorizontalCover
     * @param roomName
     * @param roomWelcome
     * @param roomRule
     * @param categoryId
     * @return
     */
    @POST
    @Path("editRoom")
    @Override
    public Result editRoom(@FormParam("roomId") Long roomId,
                           @FormParam("roomCover") String roomCover,
                           @FormParam("roomHorizontalCover") String roomHorizontalCover,
                           @FormParam("roomName") String roomName,
                           @FormParam("roomWelcome") String roomWelcome,
                           @FormParam("roomRule") String roomRule,
                           @FormParam("categoryId") Long categoryId) {
        Long userId = this.getToken().getAdminId();

        // 判断房间是否存在
        if (!rtcRoomService.isRoomExists(roomId)) {
            return new Result(GlobalApiError.OperationFailed, "未找到该房间");
        }

        // 判断权限
        int roomRole = rtcRoomService.getRoomRole(roomId, userId);
        if (roomRole < RtcRoomRole.HOST) {
            return new Result(GlobalApiError.OperationFailed, "没有权限");
        }

        // 判断类目是否有效
        if (categoryId != null && !dataService.isRtcRoomCategoryExists(categoryId)) {
            return new Result(GlobalApiError.OperationFailed, "分类无效");
        }

        if (categoryId != null) {
            // 授权语聊配置
            Long familyId = this.getToken().getFamilyId();
            AuthRtcConfig authRtcConfig = rtcUserService.getAuthRtcConfig(familyId);
            if (authRtcConfig == null) {
                return new Result(GlobalApiError.ServerError, "该功能逐步开放中，敬请期待");
            }

            // 房间分类
            String authRoomCategory = authRtcConfig.getRoomCategory();
            if (!authRoomCategory.contains(Convert.toStr(categoryId))) {
                return new Result(GlobalApiError.ServerError, "您未获得该类目授权");
            }
        }

        RtcRoomEditDTO roomEditDTO = new RtcRoomEditDTO();
        roomEditDTO.setRoomId(roomId);
        roomEditDTO.setRoomCover(roomCover);
        roomEditDTO.setRoomHorizontalCover(roomHorizontalCover);
        roomEditDTO.setRoomName(roomName);
        roomEditDTO.setRoomWelcome(roomWelcome);
        roomEditDTO.setRoomRule(roomRule);
        roomEditDTO.setCategoryId(categoryId);
        roomEditDTO.setUserId(userId);
        if (rtcRoomService.editRoom(roomEditDTO)) {
            return new Result();
        }

        return new Result(GlobalApiError.OperationFailed);
    }

    /**
     * 获取语聊列表
     *
     * @param pageIndex
     * @param pageSize
     * @param userId
     * @param roomType
     * @param roomName
     * @param categoryId
     * @param roomTitle
     * @param status
     * @return
     */
    @GET
    @Path("getRtcRoomPageList")
    @Override
    public Result getRtcRoomPageList(@QueryParam("pageIndex") int pageIndex,
                                     @QueryParam("pageSize") int pageSize,
                                     @QueryParam("userId") Long userId,
                                     @QueryParam("roomType") Integer roomType,
                                     @QueryParam("roomName") String roomName,
                                     @QueryParam("categoryId") Integer categoryId,
                                     @QueryParam("roomTitle") String roomTitle,
                                     @QueryParam("status") Integer status,
                                     @QueryParam("businessType") Integer businessType) {
        if (businessType == null) {
            businessType = 0;
        }
        return new Result(faRtcRoomService.getRtcRoomPageList(this.getToken().getFamilyId(), pageIndex, pageSize, userId, roomType, roomName, categoryId, roomTitle, status, businessType));
    }

    /**
     * 获取语聊详情
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getRtcRoomDetail")
    @Override
    public Result getRtcRoomDetail(@QueryParam("roomId") Long roomId) {
        return new Result(faRtcRoomService.getRtcRoomDetail(roomId));
    }

    /**
     * 获取房间日流水
     *
     * @param roomId
     * @param startTime
     * @param endTime
     * @return
     */
    @GET
    @Path("getRtcRoomDayStream")
    @Override
    public Result getRtcRoomDayStream(@QueryParam("roomId") Long roomId,
                                      @QueryParam("startTime") Long startTime,
                                      @QueryParam("endTime") Long endTime) {
        if (roomId == null || startTime == null || endTime == null) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        Long familyId = this.getToken().getFamilyId();
        if (!familyId.equals(rtcRoomService.getFamilyId(roomId))) {
            return new Result(GlobalApiError.ServerError, "你没有权限");
        }
        long diffTime = endTime - startTime;
        // 90天
        long ninetyDays = 90 * 24 * 3600 * 1000L;
        if (diffTime < 0 || diffTime > ninetyDays) {
            return new Result(GlobalApiError.ServerError, "时间跨度最长90天");
        }
        return new Result(rtcRoomService.getRtcRoomDayStream(roomId, startTime, endTime));
    }

    /**
     * 获取房间周流水
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getRtcRoomWeekStream")
    @Override
    public Result getRtcRoomWeekStream(@QueryParam("roomId") Long roomId) {
        if (roomId == null) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        Long familyId = this.getToken().getFamilyId();
        if (!familyId.equals(rtcRoomService.getFamilyId(roomId))) {
            return new Result(GlobalApiError.ServerError, "你没有权限");
        }
        return new Result(rtcRoomService.getRtcRoomWeekStream(roomId));
    }

    /**
     * 获取房间月流水
     *
     * @param roomId
     * @return
     */
    @GET
    @Path("getRtcRoomMonthStream")
    @Override
    public Result getRtcRoomMonthStream(@QueryParam("roomId") Long roomId) {
        if (roomId == null) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        Long familyId = this.getToken().getFamilyId();
        if (!familyId.equals(rtcRoomService.getFamilyId(roomId))) {
            return new Result(GlobalApiError.ServerError, "你没有权限");
        }
        return new Result(rtcRoomService.getRtcRoomMonthStream(roomId));
    }

    /**
     * 获取房间流水
     *
     * @param startTime
     * @param endTime
     * @return
     */
    @GET
    @Path("getRtcRoomStream")
    @Override
    public Result getRtcRoomStream(@QueryParam("startTime") Long startTime, @QueryParam("endTime") Long endTime, @QueryParam("liveType") Integer liveType) {
        if (startTime == null || endTime == null) {
            return new Result(GlobalApiError.ServerError, "缺少必要参数");
        }
        MutableResult mResult = new MutableResult();
//        mResult.put("list", rtcRoomService.getRtcRoomStream(startTime, endTime, this.getToken().getFamilyId()));
        mResult.put("list", rtcRoomService.getRtcRoomStream(startTime, endTime, 1, this.getToken().getFamilyId(), liveType));
        return mResult.build();
    }

}
